function run_file(workflowFilePath::String)
    # Parse the workflow file
    tasks, globals = parseWorkflow(workflowFilePath)

    # Get a list of unique services we need for the workflow
    unique_services = collectServices(tasks)

    # Login to image registry
    encoded_auth = registry_login("registry.nilu.no", "FACT_token", "glpat-1zG-TQx___xLsPjj2vsG")

    # Start the results container & get a connection to it
    results_container = startResultsContainer(encoded_auth)
    results_connection = get_connection(results_container)

    # Start the data containers
    startDataContainers(unique_services, encoded_auth)

    # Execute the workflow
    manage_workflow(tasks, globals, results_connection)

    # Drop the connection, stop and store the results container
    close_connection(results_connection)
    stopResultsContainer(encoded_auth)

    # Stop the data containers
    stopDataContainers(unique_services, encoded_auth)

end

function manage_workflow(tasks::Dict, globals::Dict, results_connection::MySQL.Connection)
    # Extract values from globals
    cpus = get(globals, "cpus", 1)  # Default to 1 if not specified
    mem_GB = get(globals, "mem_GB", 1)  # Default to 1 to avoid division by zero
    min_mem_GB_per_thread = get(globals, "min_mem_GB_per_thread", 1)  # Default to 1

    # Calculate the maximum number of parallel tasks
    max_parallel_tasks = min(cpus, div(mem_GB, min_mem_GB_per_thread))

    # Initialize a Dict to track completion status, and for dependencies
    task_completion_status = Dict{String, Bool}()

    # Populate the Dict with initial completion status
    for task_name in keys(tasks)
        task_completion_status[task_name] = false
    end

    # Initialize the semaphore channel to manage max number of parallel processes
    semaphore = Channel{Bool}(max_parallel_tasks)

    # Pre-fill the semaphore with tokens
    for _ = 1:max_parallel_tasks
        put!(semaphore, true)
    end
 
    # Start the tasks logic
    for (task_name, task_info) in tasks
        dependencies = task_info["dependencies"]
        dependencies_info = Dict{String, Dict}()
    
        for dep in dependencies
            dependencies_info[dep] = tasks[dep]
        end

        @async begin
            # Wait for a token from the semaphore (permission to start)
            take!(semaphore)
        
            # Wait for all dependencies to be completed
            for dep in dependencies
                while !get(task_completion_status, dep, false)  # Wait until dependency is completed
                    sleep(0.1)  # Avoid busy waiting
                end
            end
        
            # Execute the task, passing the semaphore for management within the task
            run_task(task_name, task_info, dependencies_info, results_connection, task_completion_status, semaphore)
        end

    end

    # Final waiting logic to check the completion status Dict
    for task_name in keys(tasks)
        while !task_completion_status[task_name]
            sleep(0.1)  # Polling delay
        end
        #println("Task $task_name has completed")
    end

end
